;; Comparing two numbers written in index form like 2^11 and 3^7 is
;; not difficult, as any calculator would confirm that 2^11 = 2048 3^7
;; = 2187.

;; However, confirming that 632382^518061 519432^525806 would be much
;; more difficult, as both numbers contain over three million digits.

;; Using base_exp.txt (right click and 'Save Link/Target As...'), a
;; 22K text file containing one thousand lines with a base/exponent
;; pair on each line, determine which line number has the greatest
;; numerical value.

;; NOTE: The first two lines in the file represent the numbers in the
;; example given above.

(defvar *the-file* nil)
(setf *the-file* "/home/mortis/personal/projects/project-euler.net/data-files/base_exp.txt")


(require :split-sequence)

;; (split-sequence:split-sequence #\, "foo,bar")

(defun get-pairs ()
  (with-open-file (in *the-file* :direction :input)
    (loop 
     for line = (read-line in nil nil) then (read-line in nil nil)
     while line
     collect (mapcar #'read-from-string (split-sequence:split-sequence #\, line)))))

;; (length (get-pairs))
;; (car (get-pairs))
	
'(
  (time
   (let ((max 0))
     (loop for pair in (get-pairs)
	   for base = (first pair) then (first pair)
	   for pow = (second pair) then (second pair)
	   for nval = (expt base pow) then (expt base pow)
	   repeat 3
	   do
	   (when (> nval max)
	     (format t "new max: ~a old:~a~&" nval max)
	     (setf max nval)))
     (format t "max: ~a~&" max)
     max))
)

;; ok, uncle, naieve (even w/cl/sbcl) doesn't work...

;; 2^3 = 8
;; 3^2 = 9

;;
;; 8^4 => (2*2*2)^4 =>              2^4 * 2^4 * 2^4
;; 4^8 => (2*2)^8   => 2^8 * 2^8 => 2^4 * 2^4 * 2^4 * 2^4

;; (expt 2 4)16

;;(expt 8 4) ;; 4096
;;(expt 4 8) ;; 65536
;;(/ (expt 4 8)
;;   (expt 8 4)) ;;16
;;(expt 2 8);;256
;; (* (expt 2 4) (expt 2 4)) ;;256

;; from: http://en.wikipedia.org/wiki/Exponentiating_by_squaring
;; but it's not faster than the built in expt function
(defun my-bigpow (base power)
  (cond ((= 0 power)
	 1)
	((oddp power)
	 (* base (my-bigpow (* base base)
			    (/ (- power 1) 2))))
	(t
	 (my-bigpow (* base base)
		    (/ power 2)))))

;; (time (progn (my-bigpow 987654321 10000) t))
;; (time (progn (length (format nil "~d" (expt 987654321 10000))) t))
;; 89947

;;(length (format nil "~d" (my-bigpow 987654321 10000)))

;; (time (progn (expt 75501 61609) t))
;; (progn (expt 75501 616091) t)


